@@ -3,15 +3,16 @@
 Translates pygls types back and forth with Jedi
 """
 
+import sys
 from inspect import Parameter
-from typing import Dict, List, Optional
+from typing import Dict, Iterator, List, Optional, Tuple
 
 import docstring_to_markdown
 import jedi.api.errors
 import jedi.inference.references
 import jedi.settings
 from jedi import Project, Script
-from jedi.api.classes import Completion, Name, ParamName, Signature
+from jedi.api.classes import BaseName, Completion, Name, ParamName, Signature
 from pygls.lsp.types import (
     CompletionItem,
     CompletionItemKind,
@@ -33,6 +34,17 @@ from .initialization_options import HoverDisableOptions, InitializationOptions
 from .type_map import get_lsp_completion_type, get_lsp_symbol_type
 
 
+def _jedi_debug_function(
+    color: str,  # pylint: disable=unused-argument
+    str_out: str,
+) -> None:
+    """Jedi debugging function that prints to stderr.
+
+    Simple for now, just want it to work.
+    """
+    print(str_out, file=sys.stderr)
+
+
 def set_jedi_settings(  # pylint: disable=invalid-name
     initialization_options: InitializationOptions,
 ) -> None:
@@ -47,6 +59,8 @@ def set_jedi_settings(  # pylint: disable=invalid-name
     jedi.settings.case_insensitive_completion = (
         initialization_options.jedi_settings.case_insensitive_completion
     )
+    if initialization_options.jedi_settings.debug:
+        jedi.set_debug_function(func_cb=_jedi_debug_function)
 
 
 def script(project: Optional[Project], document: Document) -> Script:
@@ -54,13 +68,20 @@ def script(project: Optional[Project], document: Document) -> Script:
     return Script(code=document.source, path=document.path, project=project)
 
 
-def lsp_range(name: Name) -> Range:
+def lsp_range(name: Name) -> Optional[Range]:
     """Get LSP range from Jedi definition.
 
     - jedi is 1-indexed for lines and 0-indexed for columns
     - LSP is 0-indexed for lines and 0-indexed for columns
     - Therefore, subtract 1 from Jedi's definition line
+
+    Not all jedi Names have their location defined.  Module attributes
+    (e.g. __name__ or __file__) have a Name that represents their
+    implicit definition, and that Name does not have a location.
     """
+    if name.line is None or name.column is None:
+        return None
+
     return Range(
         start=Position(line=name.line - 1, character=name.column),
         end=Position(
@@ -76,7 +97,11 @@ def lsp_location(name: Name) -> Optional[Location]:
     if module_path is None:
         return None
 
-    return Location(uri=module_path.as_uri(), range=lsp_range(name))
+    lsp = lsp_range(name)
+    if lsp is None:
+        return None
+
+    return Location(uri=module_path.as_uri(), range=lsp)
 
 
 def lsp_symbol_information(name: Name) -> Optional[SymbolInformation]:
@@ -95,7 +120,7 @@ def lsp_symbol_information(name: Name) -> Optional[SymbolInformation]:
     )
 
 
-def _document_symbol_range(name: Name) -> Range:
+def _document_symbol_range(name: Name) -> Optional[Range]:
     """Get accurate full range of function.
 
     Thanks <https://github.com/CXuesong> from
@@ -140,10 +165,12 @@ def lsp_document_symbols(names: List[Name]) -> List[DocumentSymbol]:
         if parent.type == "module":
             # add module-level variables to list
             results.append(symbol)
-            if name.type == "class":
-                # if they're a class, they can also be a namespace
-                _name_lookup[name] = symbol
-        elif (
+
+        if name.type in ["class", "function"]:
+            # if they're a class, they can also be a namespace
+            _name_lookup[name] = symbol
+
+        if (
             parent.type == "class"
             and name.type == "function"
             and name.name in {"__init__"}
@@ -172,11 +199,18 @@ def lsp_document_symbols(names: List[Name]) -> List[DocumentSymbol]:
                 # far as code is concerned, @property-decorated items should be
                 # considered "methods" since do more than just assign a value.
                 symbol.kind = SymbolKind.Method
-            else:
+            elif name.type != "class":
                 symbol.kind = SymbolKind.Property
             parent_symbol = _name_lookup[parent]
             assert parent_symbol.children is not None
             parent_symbol.children.append(symbol)
+        elif parent.type == "function":
+            # only show nested classes and functions to avoid excessive info
+            # could be controlled by an initialization option
+            if name.type in ["class", "function"]:
+                parent_symbol = _name_lookup[parent]
+                assert parent_symbol.children is not None
+                parent_symbol.children.append(symbol)
     return results
 
 
@@ -195,7 +229,7 @@ def lsp_diagnostic(error: jedi.api.errors.SyntaxError) -> Diagnostic:
     )
 
 
-def line_column(jedi_script: Script, position: Position) -> Dict[str, int]:
+def line_column(position: Position) -> Tuple[int, int]:
     """Translate pygls Position to Jedi's line/column.
 
     Returns a dictionary because this return result should be unpacked as a
@@ -212,20 +246,11 @@ def line_column(jedi_script: Script, position: Position) -> Dict[str, int]:
         the line is represented as a string, the `character` value represents
         the gap between the `character` and `character + 1`.
 
-        If the character value is greater than the line length it defaults back
-        to the line length.
-
     Sources:
     https://microsoft.github.io/language-server-protocol/specification#position
     https://github.com/palantir/python-language-server/pull/201/files
     """
-    lines = jedi_script._code_lines  # pylint: disable=protected-access
-    line_length = len(lines[position.line])
-    character = 1 if position.character == 0 else position.character
-    return dict(
-        line=position.line + 1,
-        column=min(character, line_length - 1 if line_length else 0),
-    )
+    return (position.line + 1, position.character)
 
 
 def line_column_range(pygls_range: Range) -> Dict[str, int]:
@@ -257,7 +282,7 @@ def compare_names(name1: Name, name2: Name) -> bool:
     return equal
 
 
-def complete_sort_name(name: Completion) -> str:
+def complete_sort_name(name: Completion, append_text: str) -> str:
     """Return sort name for a jedi completion.
 
     Should be passed to the sortText field in CompletionItem. Strings sort a-z,
@@ -267,9 +292,18 @@ def complete_sort_name(name: Completion) -> str:
     For this reason, we make sure the sort-text is just a letter and not the
     name itself.
     """
-    if name.type == "param" and name.name.endswith("="):
-        return "a"
-    return "z"
+    name_str = name.name
+    if name_str is None:
+        return "z" + append_text
+    if name.type == "param" and name_str.endswith("="):
+        return "a" + append_text
+    if name_str.startswith("_"):
+        if name_str.startswith("__"):
+            if name_str.endswith("__"):
+                return "y" + append_text
+            return "x" + append_text
+        return "w" + append_text
+    return "v" + append_text
 
 
 def clean_completion_name(name: str, char_before_cursor: str) -> str:
@@ -348,12 +382,13 @@ def clear_completions_cache() -> None:
     _MOST_RECENT_COMPLETIONS.clear()
 
 
-def lsp_completion_item(
+def lsp_completion_item(  # pylint: disable=too-many-arguments
     completion: Completion,
     char_before_cursor: str,
     enable_snippets: bool,
     resolve_eagerly: bool,
     markup_kind: MarkupKind,
+    sort_append_text: str = "",
 ) -> CompletionItem:
     """Using a Jedi completion, obtain a jedi completion item."""
     completion_name = completion.name
@@ -363,7 +398,7 @@ def lsp_completion_item(
         label=completion_name,
         filter_text=completion_name,
         kind=lsp_type,
-        sort_text=complete_sort_name(completion),
+        sort_text=complete_sort_name(completion, sort_append_text),
         insert_text=name_clean,
         insert_text_format=InsertTextFormat.PlainText,
     )
@@ -458,9 +493,9 @@ def convert_docstring(docstring: str, markup_kind: MarkupKind) -> str:
     return docstring_stripped
 
 
-_HOVER_SIGNATURE_TYPES = {"class", "instance", "function"}
+_SIGNATURE_TYPES = {"class", "function"}
 
-_HOVER_TYPE_TRANSLATION = {
+_SIGNATURE_TYPE_TRANSLATION = {
     "module": "module",
     "class": "class",
     "instance": "instance",
@@ -473,6 +508,28 @@ _HOVER_TYPE_TRANSLATION = {
 }
 
 
+def get_full_signatures(name: BaseName) -> Iterator[str]:
+    """Return the full function signature with parameters."""
+    signatures = name.get_signatures()
+    name_type = name.type
+    if not signatures:
+        if name_type not in _SIGNATURE_TYPES:
+            yield name.description
+        else:
+            name_type_trans = _SIGNATURE_TYPE_TRANSLATION[name_type]
+            yield f"{_SIGNATURE_TYPE_TRANSLATION[name_type]} {name.name}()"
+        return
+    name_type_trans = _SIGNATURE_TYPE_TRANSLATION[name_type]
+    for signature in signatures:
+        yield f"{name_type_trans} {signature.to_string()}"
+
+
+def signature_string(signature: Signature) -> str:
+    """Convert a single signature to a string."""
+    name_type_trans = _SIGNATURE_TYPE_TRANSLATION[signature.type]
+    return f"{name_type_trans} {signature.to_string()}"
+
+
 def _hover_ignore(name: Name, init: InitializationOptions) -> bool:
     """True if hover should be ignored, false otherwise.
 
@@ -505,38 +562,11 @@ def hover_text(
     name = names[0]
     if _hover_ignore(name, initialization_options):
         return None
-    name_str = name.name
-    name_type = name.type
     full_name = name.full_name
-    hover_type = _HOVER_TYPE_TRANSLATION[name_type]
-    signatures = (
-        [f"{hover_type} {s.to_string()}" for s in name.get_signatures()]
-        if name_type in _HOVER_SIGNATURE_TYPES
-        else []
-    )
     description = name.description
     docstring = name.docstring(raw=True)
-    if not signatures and name_type != "class":
-        try:
-            type_hint = name.get_type_hint()
-        except Exception:  # pylint: disable=broad-except
-            # jedi randomly raises NotImplemented, TypeError, and possibly more
-            # errors here. One example from jls -
-            # test_hover.test_hover_on_method
-            type_hint = ""
-    else:
-        type_hint = ""
-
-    if signatures:
-        header_plain = "\n".join(signatures)
-    elif name_type == "class":
-        header_plain = f"{hover_type} {name_str}"
-    elif type_hint:
-        header_plain = f"{name_str}: {type_hint}"
-    else:
-        header_plain = f"{hover_type} {name_str}"
+    header_plain = "\n".join(get_full_signatures(name))
     header = _md_python(header_plain, markup_kind)
-
     result: List[str] = []
     result.append(header)
     if docstring:
@@ -548,7 +578,7 @@ def hover_text(
         result.append("---")
         result.append(_md_python(description, markup_kind))
 
-    if full_name:
+    if full_name and name.type != "module":
         if len(result) == 1:
             result.append("---")
         result.append(
@@ -565,7 +595,7 @@ def lsp_completion_item_resolve(
 ) -> CompletionItem:
     """Resolve completion item using cached jedi completion data."""
     completion = _MOST_RECENT_COMPLETIONS[item.label]
-    item.detail = completion.description
-    docstring = convert_docstring(completion.docstring(), markup_kind)
+    item.detail = next(get_full_signatures(completion), completion.name)
+    docstring = convert_docstring(completion.docstring(raw=True), markup_kind)
     item.documentation = MarkupContent(kind=markup_kind, value=docstring)
     return item
