@@ -184,20 +184,32 @@ def completion(
     server: JediLanguageServer, params: CompletionParams
 ) -> Optional[CompletionList]:
     """Returns completion items."""
+    # pylint: disable=too-many-locals
+    snippet_disable = server.initialization_options.completion.disable_snippets
+    resolve_eagerly = server.initialization_options.completion.resolve_eagerly
+    ignore_patterns = server.initialization_options.completion.ignore_patterns
     document = server.workspace.get_document(params.text_document.uri)
     jedi_script = jedi_utils.script(server.project, document)
-    jedi_lines = jedi_utils.line_column(jedi_script, params.position)
-    completions_jedi = jedi_script.complete(**jedi_lines)
+    jedi_lines = jedi_utils.line_column(params.position)
+    completions_jedi_raw = jedi_script.complete(*jedi_lines)
+    if not ignore_patterns:
+        # A performance optimization. ignore_patterns should usually be empty;
+        # this special case avoid repeated filter checks for the usual case.
+        completions_jedi = (comp for comp in completions_jedi_raw)
+    else:
+        completions_jedi = (
+            comp
+            for comp in completions_jedi_raw
+            if not any(i.match(comp.name) for i in ignore_patterns)
+        )
     snippet_support = server.client_capabilities.get_capability(
         "text_document.completion.completion_item.snippet_support", False
     )
-    snippet_disable = server.initialization_options.completion.disable_snippets
-    resolve_eagerly = server.initialization_options.completion.resolve_eagerly
     markup_kind = _choose_markup(server)
     is_import_context = jedi_utils.is_import(
         script_=jedi_script,
-        line=jedi_lines["line"],
-        column=jedi_lines["column"],
+        line=jedi_lines[0],
+        column=jedi_lines[1],
     )
     enable_snippets = (
         snippet_support and not snippet_disable and not is_import_context
@@ -207,6 +219,9 @@ def completion(
         position=params.position,
     )
     jedi_utils.clear_completions_cache()
+    # number of characters in the string representation of the total number of
+    # completions returned by jedi.
+    total_completion_chars = len(str(len(completions_jedi_raw)))
     completion_items = [
         jedi_utils.lsp_completion_item(
             completion=completion,
@@ -214,8 +229,9 @@ def completion(
             enable_snippets=enable_snippets,
             resolve_eagerly=resolve_eagerly,
             markup_kind=markup_kind,
+            sort_append_text=str(count).zfill(total_completion_chars),
         )
-        for completion in completions_jedi
+        for count, completion in enumerate(completions_jedi)
     ]
     return (
         CompletionList(is_incomplete=False, items=completion_items)
@@ -238,12 +254,12 @@ def signature_help(
     """
     document = server.workspace.get_document(params.text_document.uri)
     jedi_script = jedi_utils.script(server.project, document)
-    jedi_lines = jedi_utils.line_column(jedi_script, params.position)
-    signatures_jedi = jedi_script.get_signatures(**jedi_lines)
+    jedi_lines = jedi_utils.line_column(params.position)
+    signatures_jedi = jedi_script.get_signatures(*jedi_lines)
     markup_kind = _choose_markup(server)
     signatures = [
         SignatureInformation(
-            label=signature.to_string(),
+            label=jedi_utils.signature_string(signature),
             documentation=MarkupContent(
                 kind=markup_kind,
                 value=jedi_utils.convert_docstring(
@@ -278,11 +294,11 @@ def definition(
     """Support Goto Definition."""
     document = server.workspace.get_document(params.text_document.uri)
     jedi_script = jedi_utils.script(server.project, document)
-    jedi_lines = jedi_utils.line_column(jedi_script, params.position)
+    jedi_lines = jedi_utils.line_column(params.position)
     names = jedi_script.goto(
+        *jedi_lines,
         follow_imports=True,
         follow_builtin_imports=True,
-        **jedi_lines,
     )
     definitions = [
         definition
@@ -309,10 +325,13 @@ def highlight(
     """
     document = server.workspace.get_document(params.text_document.uri)
     jedi_script = jedi_utils.script(server.project, document)
-    jedi_lines = jedi_utils.line_column(jedi_script, params.position)
-    names = jedi_script.get_references(**jedi_lines, scope="file")
+    jedi_lines = jedi_utils.line_column(params.position)
+    names = jedi_script.get_references(*jedi_lines, scope="file")
+    lsp_ranges = [jedi_utils.lsp_range(name) for name in names]
     highlight_names = [
-        DocumentHighlight(range=jedi_utils.lsp_range(name)) for name in names
+        DocumentHighlight(range=lsp_range)
+        for lsp_range in lsp_ranges
+        if lsp_range
     ]
     return highlight_names if highlight_names else None
 
@@ -324,10 +343,16 @@ def hover(
     """Support Hover."""
     document = server.workspace.get_document(params.text_document.uri)
     jedi_script = jedi_utils.script(server.project, document)
-    jedi_lines = jedi_utils.line_column(jedi_script, params.position)
+    jedi_lines = jedi_utils.line_column(params.position)
     markup_kind = _choose_markup(server)
+    # jedi's help function is buggy when the column is 0. For this reason, as a
+    # rote fix, we simply set the column to 1 if params.position returns column
+    # 0.
     hover_text = jedi_utils.hover_text(
-        jedi_script.help(**jedi_lines),
+        jedi_script.help(
+            line=jedi_lines[0],
+            column=1 if jedi_lines[1] == 0 else jedi_lines[1],
+        ),
         markup_kind,
         server.initialization_options,
     )
@@ -346,8 +371,8 @@ def references(
     """Obtain all references to text."""
     document = server.workspace.get_document(params.text_document.uri)
     jedi_script = jedi_utils.script(server.project, document)
-    jedi_lines = jedi_utils.line_column(jedi_script, params.position)
-    names = jedi_script.get_references(**jedi_lines)
+    jedi_lines = jedi_utils.line_column(params.position)
+    names = jedi_script.get_references(*jedi_lines)
     locations = [
         location
         for location in (jedi_utils.lsp_location(name) for name in names)
@@ -461,11 +486,9 @@ def rename(
     """Rename a symbol across a workspace."""
     document = server.workspace.get_document(params.text_document.uri)
     jedi_script = jedi_utils.script(server.project, document)
-    jedi_lines = jedi_utils.line_column(jedi_script, params.position)
+    jedi_lines = jedi_utils.line_column(params.position)
     try:
-        refactoring = jedi_script.rename(
-            new_name=params.new_name, **jedi_lines
-        )
+        refactoring = jedi_script.rename(*jedi_lines, new_name=params.new_name)
     except RefactoringError:
         return None
     changes = text_edit_utils.lsp_document_changes(
@@ -496,14 +519,14 @@ def code_action(
     document = server.workspace.get_document(params.text_document.uri)
     jedi_script = jedi_utils.script(server.project, document)
     code_actions = []
-    jedi_lines = jedi_utils.line_column(jedi_script, params.range.start)
+    jedi_lines = jedi_utils.line_column(params.range.start)
     jedi_lines_extract = jedi_utils.line_column_range(params.range)
 
     try:
         if params.range.start.line != params.range.end.line:
             # refactor this at some point; control flow with exception == bad
             raise RefactoringError("inline only viable for single-line range")
-        inline_refactoring = jedi_script.inline(**jedi_lines)
+        inline_refactoring = jedi_script.inline(*jedi_lines)
     except (RefactoringError, AttributeError, IndexError):
         inline_changes = []
     else:
